'*****************************************************************************************************************
'*                                   Propeller to CPLD comunication driver                                       *
'*                                                                                                               *
'* It uses 16 bit data through pchip to CPLD.                                                                    *
'* Colors are formed by RRRRR_GGGGG_BBBBB_X                                                                      *
'* Data could be read from a SD card.                                                                            *
'* This uses also two pins for pchip-CPLD comunication                                                           *
'* One is for clock the data into the CPLD.                                                                      *
'* And the last is for advice to the pchip, when it can send data, it is out of the visible part of the image.   *
'*                                                                                                               *
'***************************************************************************************************************** 
var
   long cog
   
PUB Start(Adrr) : Success  
   Stop
   Success :=  (Cog := cognew(@entryPoint, Adrr) + 1) ' 

PUB Stop
{{Stop toggling process, if any.}}
  if Cog
    cogstop(Cog~ - 1)

dat           
        org

entryPoint         mov     outa,#0           ' All outputs to zero
                   mov     dira,direction    ' Set pin directions

:otrobmp           or      outa,AvisoPin     ' 

                   mov     time,cnt
                   add     time,Tdelay
                   waitcnt time,Tdelay       ' Some delay to get the correct syncro between both cogs 

                   waitpeq t2,mask2
                   andn    outa,AvisoPin     ' Change of PAR address prohibited

                   mov     DataAddr,PAR      ' Begin en Ancho
                   rdlong  value,DataAddr    ' 
                   add     DataAddr,#4
                   rdlong  value1,DataAddr    ' 
                   add     DataAddr,#4
                   rdlong  value2,DataAddr    ' 
                   add     DataAddr,#4
                   rdlong  value3,DataAddr    ' 
                   add     DataAddr,#4
                   rdlong  value4,DataAddr    ' 
                   add     DataAddr,#4
                   rdlong  value5,DataAddr    ' 

                   mov     tempo,value        'check for the need to get a transparent image for color := black
                   and     tempo,maskb5
                   cmp     tempo,#0       wz
            if_z   mov     trans,#0
            if_nz  mov     trans,#1

                   mov     tempo,value        'check for the need to get a fast load full filled image
                   and     tempo,maskb6
                   cmp     tempo,#0       wz
            if_z   mov     full,#0
            if_nz  mov     full,#1
            if_nz  and     value,maskb7 
                  
                   shl     value,#8
                   mov     outa,value
                   or      outa,clockPin     ' Clock CPLD data
                   andn    outa,clockPin

                   shl     value1,#8
                   mov     outa,value1
                   or      outa,clockPin     ' Clock CPLD data
                   andn    outa,clockPin

                   shl     value2,#8
                   mov     outa,value2
                   or      outa,clockPin     ' Clock CPLD data
                   andn    outa,clockPin

                   shl     value3,#8
                   mov     outa,value3
                   or      outa,clockPin     ' Clock CPLD data
                   andn    outa,clockPin

                   or      outa,clockPin     ' Dummy Clock
                   andn    outa,clockPin

                   or      outa,clockPin     ' Dummy Clock
                   andn    outa,clockPin

                   mov     veces, value4     ' Recieve the total of pixeles of this frame or "image"

                   or      outa,AvisoPin     ' 

                   mov     totpix,value5
                   mov     tempo,veces
                   and     tempo,maskb3
                   cmp     tempo,#128       wz
            if_z   jmp     #:especial

                   test    full,#1          wz
            if_nz  jmp     #:fastfull           'Fast frame load
                   
'**********************************************************************************************************
'**********************************************************************************************************
' Parte para mandar al VGA imagenes de 640x480
                   mov     time,cnt
                   add     time,#400
                   waitcnt time,#400         ' Some delay to get the correct syncro between both cogs

                   waitpeq t2,mask2
                   andn    outa,AvisoPin     ' Change of PAR address data, prohibited


:mandadata         mov     dount,veces      ' 
:copyToDisplay     mov     count,pixperframe       ' 
                   mov     DataAddr,PAR 
:pixelLoop         rdlong  long1,DataAddr    ' 
                   add     DataAddr,#4

                   mov     tempo,long1
                   and     tempo,maskb1
                   shl     tempo,#8
                   mov     outa,tempo
                   waitpeq t1,mask           'Wait for syncro H or V to be on in VGA signal
                   or      outa,clockPin     'Clock in this data into CPLD
                   andn    outa,clockPin
                   test    trans,#1       wz
            if_nz  or      outa,clockPin     'Clock in this data into CPLD
            if_nz  andn    outa,clockPin

                   mov     tempo,long1
                   and     tempo,maskb2
                   shr     tempo,#8
                   mov     outa,tempo
                   waitpeq t1,mask           'Wait for syncro H or V to be on in VGA signal
                   or      outa,clockPin     'Clock in this data into CPLD
                   andn    outa,clockPin
                   test    trans,#1       wz
            if_nz  or      outa,clockPin     'Clock in this data into CPLD
            if_nz  andn    outa,clockPin

                   djnz    count,#:pixelLoop ' Go back for next pixel

                   or      outa,AvisoPin     ' 
                   waitpeq t2,mask2
                   andn    outa,AvisoPin     ' Change of PAR address data, prohibited

                   djnz    dount,#:copyToDisplay ' Go back for next pixel

:dummy_end         mov     outa,#0
                   or      outa,clockPin     ' Clock CPLD data
                   andn    outa,clockPin
                   jmp     #:otrobmp
'**********************************************************************************************************
'**********************************************************************************************************
' For sent to CPLD images smaller than 640x480

:especial          mov     time,cnt
                   add     time,Tdelay2
                   waitcnt time,Tdelay2       ' Some delay to get the correct syncro between both cogs

                   waitpeq t2,mask2
                   andn    outa,AvisoPin     ' Change of PAR address data, prohibited
                   mov     contador,#0       ' Byte sent counter
                   mov     long_counter,#0   ' longs recieve counter

:NpixelLoop        mov     DataAddr,PAR 
:spixelLoop        rdlong  long1,DataAddr    ' 
                   add     DataAddr,#4
                   add     long_counter,#1   ' Each time recieve one long more
                   
                   mov     tempo,long1
                   and     tempo,maskb1
                   shl     tempo,#8
                   mov     outa,tempo
                   waitpeq t1,mask           'Wait for syncro H or V to be on in VGA signal
                   or      outa,clockPin     'Clock in this data into CPLD
                   andn    outa,clockPin
                   test    trans,#1       wz
            if_nz  or      outa,clockPin     'Clock in this data into CPLD
            if_nz  andn    outa,clockPin

                   add     contador,#1
                   cmp     contador,totpix  wz
            if_z   jmp     #:sdummy_end 

                   mov     tempo,long1
                   and     tempo,maskb2
                   shr     tempo,#8
                   mov     outa,tempo
                   waitpeq t1,mask           'Wait for syncro H or V to be on in VGA signal
                   or      outa,clockPin     'Clock in this data into CPLD
                   andn    outa,clockPin
                   test    trans,#1       wz
            if_nz  or      outa,clockPin     'Clock in this data into CPLD
            if_nz  andn    outa,clockPin

                   add     contador,#1
                   cmp     contador,totpix  wz
            if_z   jmp     #:sdummy_end 

                   cmp     long_counter,max_longs  wz
            if_z   jmp     #:swait_abit 
                   jmp     #:spixelLoop

:swait_abit        or      outa,AvisoPin     ' 

                   mov     time,cnt
                   add     time,Tdelay2
                   waitcnt time,Tdelay2      ' Some delay to get the correct syncro between both cogs

                   waitpeq t2,mask2
                   andn    outa,AvisoPin     ' Change of PAR address data, prohibited
                   mov     long_counter,#0   ' longs recieve counter
                   jmp     #:NpixelLoop


:sdummy_end        mov     outa,#0
                   or      outa,clockPin     ' Clock CPLD data
                   andn    outa,clockPin
                   jmp     #:otrobmp
'**********************************************************************************************************
'**********************************************************************************************************
'Fast data load for frames of the same filled color
:fastfull
                   mov     time,cnt
                   add     time,#400
                   waitcnt time,#400         ' Some delay to get the correct syncro between both cogs

                   waitpeq t2,mask2
                   andn    outa,AvisoPin     ' Change of PAR address data, prohibited

                   mov     count,totpix       ' 
                   mov     DataAddr,PAR 
                   rdlong  long1,DataAddr    ' 

                   mov     tempo,long1
                   and     tempo,maskb1
                   shl     tempo,#8
                   mov     outa,tempo
:fastsend          waitpeq t1,mask           'Wait for syncro H or V to be on in VGA signal
                   or      outa,clockPin     'Clock in this data into CPLD
                   andn    outa,clockPin
                   djnz    totpix,#:fastsend  ' Go back for next same pixel

                   or      outa,AvisoPin     ' 
                   waitpeq t2,mask2
                   andn    outa,AvisoPin     ' Change of PAR address data, prohibited

                   mov     outa,#0
                   or      outa,clockPin     ' Clock CPLD data
                   andn    outa,clockPin
                   jmp     #:otrobmp

'**********************************************************************************************************
'**********************************************************************************************************


direction          long    %00000000_11111111_11111111_10010000   'corrected dirs for my PCB
ClockPin           long    %00000000_00000000_00000000_10000000
AvisoPin           long    %00000000_00000000_00000000_00010000
t1                 long    1<<5
mask               long    1<<5
t2                 long    1<<30
mask2              long    1<<30
maskb1             long    %00000000_00000000_11111111_11111111
maskb2             long    %11111111_11111111_00000000_00000000
maskb3             long    %00000000_00000000_00000000_10000000
maskb4             long    %00000000_00000000_00000000_01111111
maskb5             long    %00000000_00000000_10000000_00000000
maskb6             long    %00000000_00000000_01000000_00000000
maskb7             long    %00000000_00000000_10111111_11111111
max_longs          long    1536
TDelay             long    200
TDelay2            long    2000
pixperframe        long    1536


value              res     1  ' 
value1             res     1  '
value2             res     1  ' 
value3             res     1  ' 
value4             res     1  ' 
value5             res     1  ' 

DataAddr           res     1
long1              res     1
tempo              res     1
contador           res     1
totpix             res     1
long_counter       res     1
time               res     1
veces              res     1
count              res     1
dount              res     1
trans              res     1
full               res     1

 
'